@@ -1598,7 +1598,7 @@ concat_ident (const char *prefix, const gchar *ident)
  *
  * For example the <![CDATA["test.\"ATable\""]]> string will result in the array: <![CDATA[{"test", "\"ATable\"", NULL}]]>
  *
- * Returns: a new array of strings, or NULL (use g_strfreev() to free the returned array)
+ * Returns: a new %NULL-terminated array of strings, or NULL (use g_strfreev() to free the returned array)
  */
 gchar **
 gda_sql_identifier_split (const gchar *id)
@@ -2146,7 +2146,27 @@ gda_dsn_split (const gchar *string, gchar **out_dsn, gchar **out_username, gchar
  * Extract the provider, connection parameters, username and password from @string. 
  * in @string, the various parts are strings
  * which are expected to be encoded using an RFC 1738 compliant encoding. If they are specified, 
- * the returned provider, username and password strings are correclty decoded.
+ * the returned provider, username and password strings are correctly decoded.
+ *
+ * For example all the following connection strings:
+ * <programlisting><![CDATA[
+PostgreSQL://meme:pass@DB_NAME=mydb;HOST=server
+PostgreSQL://meme@DB_NAME=mydb;HOST=server;PASSWORD=pass
+PostgreSQL://meme@DB_NAME=mydb;PASSWORD=pass;HOST=server
+PostgreSQL://meme@PASSWORD=pass;DB_NAME=mydb;HOST=server
+PostgreSQL://DB_NAME=mydb;HOST=server;USERNAME=meme;PASSWORD=pass
+PostgreSQL://DB_NAME=mydb;HOST=server;PASSWORD=pass;USERNAME=meme
+PostgreSQL://DB_NAME=mydb;USERNAME=meme;PASSWORD=pass;HOST=server
+PostgreSQL://PASSWORD=pass;USERNAME=meme;DB_NAME=mydb;HOST=server
+PostgreSQL://:pass@USERNAME=meme;DB_NAME=mydb;HOST=server
+PostgreSQL://:pass@DB_NAME=mydb;HOST=server;USERNAME=meme]]></programlisting>
+ *
+ * will return the following new strings (double quotes added here to delimit strings):
+ * <programlisting><![CDATA[
+out_cnc_params: "DB_NAME=mydb;HOST=server"
+out_provider: "PostgreSQL"
+out_username: "meme"
+out_password: "pass"]]></programlisting>
  */
 void
 gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar **out_provider, 
@@ -2195,6 +2215,46 @@ gda_connection_string_split (const gchar *string, gchar **out_cnc_params, gchar
 	if (!*out_cnc_params)
 		*out_cnc_params = g_strdup (ap);
 
+	if (*out_cnc_params) {
+		gchar *pos;
+
+		pos = g_strrstr (*out_cnc_params, "USERNAME=");
+		if (pos) {
+			for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
+			if (ptr != pos + 9)
+				*out_username = g_strndup (pos + 9, ptr - (pos + 9));
+			gchar *tmp;
+			gint lastpos;
+			tmp = g_strndup (*out_cnc_params, pos - *out_cnc_params);
+			*out_cnc_params = g_strconcat (tmp,
+						       (ptr ? (ptr + 1 ? ptr + 1 : ptr) : ""),
+						       NULL);
+			g_free (tmp);
+			tmp = *out_cnc_params;
+			lastpos = strlen(tmp) - 1;
+			if (tmp [lastpos] == ';')
+				tmp [lastpos] = 0;
+		}
+		
+		pos = g_strrstr (*out_cnc_params, "PASSWORD=");
+		if (pos) {
+			for (ptr = pos + 9; ptr && *ptr != '\0' && *ptr != ';'; ptr++);
+			if (ptr != pos + 9)
+				*out_password = g_strndup (pos + 9, ptr - (pos + 9));
+			gchar *tmp;
+			gint lastpos;
+			tmp = g_strndup (*out_cnc_params, pos - *out_cnc_params);
+			*out_cnc_params = g_strconcat (tmp,
+						       (ptr ? (ptr + 1 ? ptr + 1 : ptr) : ""),
+						       NULL);
+			g_free (tmp);
+			tmp = *out_cnc_params;
+			lastpos = strlen(tmp) - 1;
+			if (tmp [lastpos] == ';')
+				tmp [lastpos] = 0;
+		}
+	}
+
 	/* RFC 1738 decode provider, username and password strings */
 	gda_rfc1738_decode (*out_provider);
 	gda_rfc1738_decode (*out_username);
